#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

A Jumpcloud command line client
"""

from .version import __version__, __release__  # noqa
