#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. currentmodule:: jccli.version
.. moduleauthor:: zaro0508 <zaro0508@gmail.com>

This module contains project version information.
"""

__version__ = "0.0.11"  #: the working version
__release__ = "0.0.11"  #: the release version
